package com.qmth.themis.backend.api;

import javax.annotation.Resource;

import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.themis.business.service.TIeReportService;
import com.qmth.themis.common.util.Result;
import com.qmth.themis.common.util.ResultUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;

/** 监考报表
 * @Description: 
 * @Author: xiatian
 * @Date: 2020-08-28
 */
@Api(tags = "监考报表")
@RestController
@RequestMapping("/${prefix.url.admin}/report")
public class TIeReportController {
    
    @Resource
    private TIeReportService reportService;

    @ApiOperation(value = "考试概览")
    @RequestMapping(value = "/exam_view", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "结果信息")})
    public Result examView(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                             @ApiParam(value = "考场场次id", required = false) @RequestParam Long examActivityId,
                             @ApiParam(value = "虚拟考场代码", required = false) @RequestParam String roomCode,
                             @ApiParam(value = "科目编码", required = false) @RequestParam String courseCode,
                             @ApiParam(value = "姓名", required = false) @RequestParam String name,
                             @ApiParam(value = "唯一码", required = false) @RequestParam String identity) {
        return ResultUtil.ok(reportService.examView(examId, examActivityId, roomCode, courseCode, name, identity));
    }
    
    @ApiOperation(value = "情况统计")
    @RequestMapping(value = "/exam_view_count", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "结果信息")})
    public Result examViewCount(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                             @ApiParam(value = "考场场次id", required = false) @RequestParam Long examActivityId,
                             @ApiParam(value = "虚拟考场代码", required = false) @RequestParam String roomCode,
                             @ApiParam(value = "科目编码", required = false) @RequestParam String courseCode,
                             @ApiParam(value = "姓名", required = false) @RequestParam String name,
                             @ApiParam(value = "唯一码", required = false) @RequestParam String identity,
                             @ApiParam(value = "分页页码", required = true) @RequestParam int pageNumber,
                             @ApiParam(value = "分页数", required = true) @RequestParam int pageSize) {
        return ResultUtil.ok(reportService.examViewCount(examId, examActivityId, roomCode, courseCode, name, identity,pageNumber,pageSize));
    }
    
    
    @ApiOperation(value = "缺考名单")
    @RequestMapping(value = "/exam_deficiency_list", method = RequestMethod.POST)
    @ApiResponses({@ApiResponse(code = 200, message = "结果信息")})
    public Result examDeficiencyList(@ApiParam(value = "考试id", required = true) @RequestParam Long examId,
                             @ApiParam(value = "考场场次id", required = false) @RequestParam Long examActivityId,
                             @ApiParam(value = "虚拟考场代码", required = false) @RequestParam String roomCode,
                             @ApiParam(value = "科目编码", required = false) @RequestParam String courseCode,
                             @ApiParam(value = "姓名", required = false) @RequestParam String name,
                             @ApiParam(value = "唯一码", required = false) @RequestParam String identity,
                             @ApiParam(value = "分页页码", required = true) @RequestParam int pageNumber,
                             @ApiParam(value = "分页数", required = true) @RequestParam int pageSize) {
        return ResultUtil.ok(reportService.examDeficiencyList(examId, examActivityId, roomCode, courseCode, name, identity,pageNumber,pageSize));
    }
}
