package com.qmth.themis.business.service.impl;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import com.qmth.themis.business.cache.ExamActivityRecordCacheUtil;
import com.qmth.themis.business.cache.ExamRecordCacheUtil;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.enums.ExamRecordStatusEnum;
import com.qmth.themis.business.service.CommonService;
import com.qmth.themis.business.service.TOeExamRecordService;
import com.qmth.themis.business.util.RedisUtil;

/**
 * @Description:
 * @Author: xiatian
 * @Date: 2020-08-04
 */
@Service
public class CommonServiceImpl implements CommonService {
	private final static Logger log = LoggerFactory.getLogger(CommonService.class);

	@Resource
	TOeExamRecordService examRecordService;
	@Resource
	RedisUtil redisUtil;

	@Override
	public void persisted(Long recordId) {
    	ExamRecordStatusEnum status=ExamRecordCacheUtil.getStatus(recordId);
    	if(!ExamRecordStatusEnum.FINISHED.equals(status)) {
    		return;
    	}
    	Double score=ExamRecordCacheUtil.getObjectiveScore(recordId);
    	if(score==null) {
    		return;
    	}
		String lockKey = SystemConstant.REDIS_LOCK_EXAM_RECORD_PERSISTED_PREFIX + recordId;
		Boolean lock = redisUtil.lock(lockKey, SystemConstant.REDIS_CACHE_TIME_OUT);
		if (!lock) {
			log.debug("persisted doing,request ignore");
			return;
		}
		try {
			examRecordService.saveDataByCache(recordId);
			// 清除缓存
			ExamActivityRecordCacheUtil.removeActivityRecordCache(ExamRecordCacheUtil.getExamActivityId(recordId), recordId);
			redisUtil.delete(RedisKeyHelper.examRecordCacheKey(recordId));
			redisUtil.delete(RedisKeyHelper.examAnswerKey(recordId));
			redisUtil.delete(RedisKeyHelper.audioLeftPlayCountKey(recordId));
			redisUtil.delete(RedisKeyHelper.livenessVerifyCacheKey(recordId));
			redisUtil.delete(RedisKeyHelper.faceVerifyCacheKey(recordId));
		} finally {
			redisUtil.releaseLock(lockKey);
		}
	}

}
