package com.qmth.themis.exam.websocket;

import com.alibaba.fastjson.JSONObject;
import com.google.gson.Gson;
import com.qmth.themis.business.cache.RedisKeyHelper;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.dto.WebsocketDto;
import com.qmth.themis.business.entity.TBSession;
import com.qmth.themis.business.enums.MonitorStatusSourceEnum;
import com.qmth.themis.business.enums.MonitorVideoSourceEnum;
import com.qmth.themis.business.enums.WebsocketTypeEnum;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.business.util.WebsocketUtil;
import com.qmth.themis.common.enums.ExceptionResultEnum;
import com.qmth.themis.common.exception.BusinessException;
import com.qmth.themis.common.signature.SignatureInfo;
import com.qmth.themis.common.signature.SignatureType;
import com.qmth.themis.exam.websocketTemplete.WebSocketMobileMessageTemplete;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.lang.reflect.Method;
import java.net.InetSocketAddress;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

/**
 * @Description: websocket移动端
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/10
 */
@ServerEndpoint("/mobile")
@Component
public class WebSocketMobileServer {
    private final static Logger log = LoggerFactory.getLogger(WebSocketMobileServer.class);
    private volatile static ConcurrentHashMap<Long, WebSocketMobileServer> webSocketMap = new ConcurrentHashMap<>();
    /**
     * 与某个客户端的连接会话，需要通过它来给客户端发送数据
     */
    private Session session = null;
    private String sessionId = null, ip = null;
    private Long recordId = null;
    private String platform = null, deviceId = null, Authorization = null;
    private Long time = null;
    private RedisUtil redisUtil;
    private Long updateTime = null;
    private Map<String, Object> tranMap = null;
    private MonitorVideoSourceEnum source = null;

    /**
     * 连接建立成功调用的方法
     */
    @OnOpen
    public void onOpen(Session session) {
        Map<String, List<String>> mapParameter = session.getRequestParameterMap();
        if (Objects.isNull(mapParameter)) {
            throw new BusinessException(ExceptionResultEnum.PARAMS_ILLEGALITY);
        }
        log.info("mapParameter:{}", JacksonUtil.parseJson(mapParameter));
        log.info("uri:{}", session.getRequestURI());
        if (Objects.isNull(mapParameter.get("platform")) || mapParameter.get("platform").size() == 0) {
            throw new BusinessException(ExceptionResultEnum.PLATFORM_INVALID);
        }
        this.platform = String.valueOf(mapParameter.get("platform").get(0));
        if (Objects.isNull(mapParameter.get("deviceId")) || mapParameter.get("deviceId").size() == 0) {
            throw new BusinessException(ExceptionResultEnum.DEVICE_ID_INVALID);
        }
        this.deviceId = String.valueOf(mapParameter.get("deviceId").get(0));
        this.Authorization = String.valueOf(mapParameter.get("Authorization").get(0));
        this.time = Long.parseLong(String.valueOf(mapParameter.get("time").get(0)));
        this.recordId = Long.parseLong(String.valueOf(mapParameter.get("recordId").get(0)));
        this.source = MonitorVideoSourceEnum.valueOf(mapParameter.get("source").get(0));
        String method = SystemConstant.GET;
        final SignatureInfo info = SignatureInfo
                .parse(Authorization);
        if (Objects.nonNull(info) && info.getType() == SignatureType.TOKEN) {
            String sessionId = info.getInvoker();
            redisUtil = SpringContextHolder.getBean(RedisUtil.class);
            TBSession tbSession = (TBSession) redisUtil.getUserSession(sessionId);
            if (Objects.isNull(tbSession)) {
                throw new BusinessException(ExceptionResultEnum.LOGIN_NO);
            } else {
                if (info.validate(tbSession.getAccessToken()) && info.getTimestamp() < tbSession.getExpireTime().getTime()
                        && platform.equalsIgnoreCase(tbSession.getPlatform()) && Objects.equals(deviceId, tbSession.getDeviceId())) {
                    this.session = session;
                    session.setMaxIdleTimeout(SystemConstant.WEBSOCKET_MAX_TIME_OUT);
                    this.sessionId = tbSession.getId();
                    if (webSocketMap.containsKey(this.recordId)) {
                        webSocketMap.remove(this.recordId);
                        webSocketMap.put(this.recordId, this);
                        //加入set中
                    } else {
                        webSocketMap.put(this.recordId, this);
                        //加入set中
                        addOnlineCount();
                        //在线数加1
                    }
                    log.info("用户连接:" + this.sessionId + ",当前在线人数为:" + getOnlineCount());
                    InetSocketAddress addr = (InetSocketAddress) WebsocketUtil.getFieldInstance(this.session.getAsyncRemote(), "base#socketWrapper#socket#sc#remoteAddress");
                    this.ip = addr.toString().replace("/", "").split(":")[0];
//                    this.sendMessage("ip[" + this.ip + "]连接成功");
                    log.info("ip[:{}]连接成功", this.ip);
                    tranMap = new HashMap<>();
                    tranMap.put("recordId", this.recordId);
                    tranMap.put("deviceId", this.deviceId);
                    tranMap.put("ip", this.ip);
                    this.updateTime = System.currentTimeMillis();
                    tranMap.put("updateTime", this.updateTime);
                } else {
                    throw new BusinessException(ExceptionResultEnum.AUTHORIZATION_ERROR);
                }
            }
        } else {
            throw new BusinessException(ExceptionResultEnum.AUTHORIZATION_ERROR);
        }
    }

    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose() {
        log.info("onClose is come in");
        if (webSocketMap.containsKey(this.recordId)) {
            webSocketMap.remove(this.recordId);
            //从set中删除
            subOnlineCount();
            //判断是否是正常退出
            Map<String, Object> objectMap = redisUtil.getHashEntries(RedisKeyHelper.examRecordCacheKey(recordId));
            objectMap.put(SystemConstant.MONITOR_STATUS_ + this.source.name(), MonitorStatusSourceEnum.STOP);
            redisUtil.setForHash(RedisKeyHelper.examRecordCacheKey(recordId), objectMap);
            ConcurrentHashMap<Long, WebSocketOeServer> webSocketMap = WebSocketOeServer.getWebSocketMap();
            if (Objects.nonNull(webSocketMap.get(recordId))) {
                WebSocketOeServer webSocketOeServer = webSocketMap.get(recordId);
                Map map = new HashMap<>();
                map.put(SystemConstant.RECORD_ID, recordId);
                map.put("monitorVideoSource", this.source.name());
                WebsocketDto websocketDto = new WebsocketDto(WebsocketTypeEnum.MOBILE_MONITOR_STOP.name(), map);
                webSocketOeServer.sendMessage(websocketDto);
            }
        }
        log.info("用户退出:{},当前在线人数为:{},updateTime:{}", this.sessionId, getOnlineCount(), this.updateTime);
    }

    /**
     * 收到客户端消息后调用的方法
     *
     * @param message
     * @param session
     */
    @OnMessage
    public void onMessage(String message, Session session) {
        //可以群发消息
        //消息保存到数据库、redis
        if (Objects.nonNull(message) && session.isOpen()) {
            try {
                //解析发送的报文
                JSONObject jsonObject = JSONObject.parseObject(message);
                log.info("onMessage:{}", jsonObject.toJSONString());
                if (Objects.nonNull(jsonObject)) {
                    WebSocketMobileMessageTemplete webSocketMobileMessageTemplete = SpringContextHolder.getBean(WebSocketMobileMessageTemplete.class);
                    Gson gson = new Gson();
                    WebsocketDto websocketDto = gson.fromJson(gson.toJson(jsonObject), WebsocketDto.class);
                    jsonObject.getJSONObject("body").put("recordId", this.recordId);
                    websocketDto.setBody(jsonObject.getJSONObject("body"));
                    Method method = webSocketMobileMessageTemplete.getClass().getDeclaredMethod(WebsocketTypeEnum.valueOf(websocketDto.getType()).getDesc(), String.class);
                    WebsocketDto result = (WebsocketDto) method.invoke(webSocketMobileMessageTemplete, String.valueOf(websocketDto.getBody()));
                    this.sendMessage(result);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
    }

    /**
     * 错误
     *
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error) {
        log.error("用户错误:{},原因:{}", this.sessionId, error.getMessage());
        error.printStackTrace();
        throw new BusinessException(error.getMessage());
    }

    /**
     * 实现服务器主动推送
     *
     * @param message
     * @throws IOException
     */
    public void sendMessage(Object message) {
        log.info("message:{}", JacksonUtil.parseJson(message));
        this.session.getAsyncRemote().sendText(JacksonUtil.parseJson(message));
        this.updateTime = System.currentTimeMillis();
    }

    /**
     * 获取在线人数
     *
     * @return
     */
    public synchronized int getOnlineCount() {
        Object o = redisUtil.get(SystemConstant.WEBSOCKET_OE_ONLINE_COUNT);
        return Objects.isNull(o) ? 0 : (int) o;
    }

    /**
     * 在线人数加一
     */
    public synchronized void addOnlineCount() {
        if (redisUtil.lock(SystemConstant.REDIS_LOCK_WEBSOCKET_PREFIX + this.sessionId, SystemConstant.REDIS_LOCK_WEBSOCKET_TIME_OUT)) {
            Object o = redisUtil.get(SystemConstant.WEBSOCKET_OE_ONLINE_COUNT);
            int count = 0;
            if (Objects.nonNull(o)) {
                count = (int) o;
            }
            count++;
            redisUtil.set(SystemConstant.WEBSOCKET_OE_ONLINE_COUNT, count);
        }
    }

    /**
     * 在线人数减一
     */
    public synchronized void subOnlineCount() {
        if (redisUtil.lock(SystemConstant.REDIS_LOCK_WEBSOCKET_PREFIX + this.sessionId, SystemConstant.REDIS_LOCK_WEBSOCKET_TIME_OUT)) {
            Object o = redisUtil.get(SystemConstant.WEBSOCKET_OE_ONLINE_COUNT);
            int count = 0;
            if (Objects.nonNull(o)) {
                count = (int) o;
            }
            count--;
            redisUtil.set(SystemConstant.WEBSOCKET_OE_ONLINE_COUNT, count < 0 ? 0 : count);
        }
    }

    public static ConcurrentHashMap<Long, WebSocketMobileServer> getWebSocketMap() {
        return webSocketMap;
    }

    public Long getRecordId() {
        return recordId;
    }
}

