package com.qmth.themis.mq.templete.impl;

import com.alibaba.fastjson.JSONObject;
import com.qmth.themis.business.constant.SpringContextHolder;
import com.qmth.themis.business.constant.SystemConstant;
import com.qmth.themis.business.util.JacksonUtil;
import com.qmth.themis.business.util.RedisUtil;
import com.qmth.themis.common.contanst.Constants;
import com.qmth.themis.business.dto.MqDto;
import com.qmth.themis.mq.service.MqLogicService;
import com.qmth.themis.mq.templete.Concurrently;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyContext;
import org.apache.rocketmq.client.consumer.listener.ConsumeConcurrentlyStatus;
import org.apache.rocketmq.common.message.MessageExt;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @Description: mq 延时消息监听 websocket超时退出 并行消费监听
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2020/7/28
 */
@Service
public class WebsocketUnNormalConcurrentlyImpl implements Concurrently {
    private final static Logger log = LoggerFactory.getLogger(WebsocketUnNormalConcurrentlyImpl.class);

    @Override
    public ConsumeConcurrentlyStatus consumeMessage(List<MessageExt> msgs, ConsumeConcurrentlyContext consumeConcurrentlyContext) {
        RedisUtil redisUtil = SpringContextHolder.getBean(RedisUtil.class);
        MqLogicService mqLogicService = SpringContextHolder.getBean(MqLogicService.class);
        MqDto mqDto = null;
        try {
            long threadId = Thread.currentThread().getId();
            String threadName = Thread.currentThread().getName();
            for (MessageExt messageExt : msgs) {
                log.info(":{}-:{} websocket unnormal Consumer重试次数:{}", threadId, threadName, messageExt.getReconsumeTimes());
                String body = new String(messageExt.getBody(), Constants.CHARSET_NAME);
                log.info("body:{}", body);
                JSONObject jsonObject = JSONObject.parseObject(body);
                Map properties = (Map) jsonObject.get("properties");
                mqDto = JSONObject.toJavaObject(JSONObject.parseObject(String.valueOf(properties.get(SystemConstant.MQDTO_OBJ))), MqDto.class);
                log.info(":{}-:{} websocket unnormal Consumer接收到的消息:{}", threadId, threadName, JacksonUtil.parseJson(mqDto));
                int reconsumeTime = messageExt.getReconsumeTimes();
                if (reconsumeTime >= SystemConstant.MAXRECONSUMETIMES) {
                    mqLogicService.execMqMaxReconsumeTime(mqDto, SystemConstant.MQ_DELAY_TOPIC_BUFFER_LIST);
                } else {
                    if (Objects.nonNull(mqDto.getAck()) && mqDto.getAck().intValue() != SystemConstant.STANDARD_ACK_TYPE && Objects.nonNull(SystemConstant.MQ_DELAY_TOPIC_BUFFER_LIST) && redisUtil.lock(SystemConstant.REDIS_LOCK_MQ_PREFIX + mqDto.getId(), SystemConstant.REDIS_LOCK_MQ_TIME_OUT)) {
                        log.info(":{}-:{} 插入学生轨迹日志", threadId, threadName);
                        mqLogicService.execMqWebsocketUnNormalLogic(mqDto, SystemConstant.MQ_DELAY_TOPIC_BUFFER_LIST);
                        return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;
                    } else {
                        return ConsumeConcurrentlyStatus.RECONSUME_LATER;//重试
                    }
                }
            }
        } catch (Exception e) {
            log.error("mq 延时消息监听，消息消费出错", e);
            e.printStackTrace();
            return ConsumeConcurrentlyStatus.RECONSUME_LATER;//重试
        } finally {
            if (Objects.nonNull(mqDto)) {
                redisUtil.releaseLock(SystemConstant.REDIS_LOCK_MQ_PREFIX + mqDto.getId());
            }
        }
        return ConsumeConcurrentlyStatus.CONSUME_SUCCESS;//成功
    }
}
